/**************************************************************************
 *
 * FILE  buffer.c
 * Copyright (c) 2017 Daniel Kahlin <daniel@kahlin.net>
 * Written by Daniel Kahlin <daniel@kahlin.net>
 *
 * DESCRIPTION
 *   handling of buffers
 *
 ******/
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include "global.h"
#include "buffer.h"
#include "utils.h"


/**************************************************************************
 *
 * NAME  create_buffer, destroy_buffer
 *
 * DESCRIPTION
 *   Create/destroy buffer.
 *
 ******/
Buffer *create_buffer(size_t size)
{
    Buffer *bf;

    bf = safe_malloc(sizeof(Buffer), "Buffer");
    bf->buf = safe_malloc(size, "Buffer buf");
    bf->size = size;

    bf->pos = 0;
    bf->len = 0;

    return bf;
}

void destroy_buffer(Buffer *bf)
{
    free(bf->buf);
    free(bf);
}



/**************************************************************************
 *
 * NAME  create_buffer_from_file
 *
 * DESCRIPTION
 *   Create a new buffer from file.
 *
 ******/
Buffer *create_buffer_from_file(const char *name)
{
    Buffer *bf;
    size_t len;
    FILE *fp;

    fp = fopen(name, "rb");
    if (!fp) {
	panic("couldn't open file for reading");
    }

    fseek(fp, 0, SEEK_END);
    len = ftell(fp);
    bf = create_buffer(len);

    fseek(fp, 0, SEEK_SET);

    uint8_t *b = bf->buf;
    int l = len;
    while (l > 0) {
	size_t n;
	n = fread(b, 1, l, fp);
	b += n;
	l -= n;
    }

    fclose(fp);

    bf->len = len;

    return bf;
}


/**************************************************************************
 *
 * NAME  write_buffer_to_file
 *
 * DESCRIPTION
 *   Write buffer contents to file.
 *
 ******/
void write_buffer_to_file(Buffer *bf, const char *name)
{
    FILE *fp;

    fp = fopen(name, "wb");
    if (!fp) {
	panic("couldn't open file for writing");
    }

    fwrite(bf->buf, 1, bf->len, fp);

    fclose(fp);

}



/* this below should probably go to a separate file */
/**************************************************************************
 *
 * NAME  create_prgobj, create_prgobj_from_file, destroy_prgobj
 *
 * DESCRIPTION
 *   Create/destroy prgobj.
 *
 ******/
PrgObj *create_prgobj(size_t size)
{
    PrgObj *po;
    po = safe_malloc(sizeof(PrgObj), "PrgObj");
    po->bf = create_buffer(size);
    po->jmp = 0xfce2;
    po->mem = 0x37;
    po->sei = 0;

    return po;
}

PrgObj *create_prgobj_from_file(const char *name)
{
    PrgObj *po;
    po = safe_malloc(sizeof(PrgObj), "PrgObj");
    po->bf = create_buffer_from_file(name);
    po->jmp = 0xfce2;
    po->mem = 0x37;
    po->sei = 0;

    return po;
}

void write_prgobj_to_file(PrgObj *po, const char *name)
{
    write_buffer_to_file(po->bf, name);
}

void destroy_prgobj(PrgObj *po)
{
    destroy_buffer(po->bf);
    free(po);
}

/* eof */
